package com.student.details.services;
import com.student.details.exceptions.ResourceNotFoundException;
import com.student.details.models.Student;
import com.student.details.repositories.StudentRepository;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.*;

@Service
public class StudentService {

    @Autowired
    StudentRepository studentRepository;

    public JSONObject fetchService(String stringUrl) {
        System.out.println("demo api");
        try {
            URL url = new URL(stringUrl);
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            conn.setRequestMethod("GET");
            conn.connect();

            int responseCode = conn.getResponseCode();
            if (responseCode != 200) {
                throw new RuntimeException("" + responseCode);
            } else {
                String res = "";
                Scanner scanner = new Scanner(url.openStream());

                while (scanner.hasNext()) {
                    String nextVal = scanner.nextLine();
                    res += nextVal;
                }

                JSONParser parser = new JSONParser();
                JSONObject json = (JSONObject) parser.parse(res);
                return json;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return new JSONObject();
    }

    public List<Student> findAllStudents() {
        return studentRepository.findAll();
    }

    public Student addStudent(Student student) {
        return studentRepository.save(student);
    }

    public Optional<Student> getStudentById(Long studentId) {
        return studentRepository.findById(studentId);
    }

    public Optional<Student> getStudentByEmail(String email) {
        return studentRepository.findStudentByEmail(email);
    }

    public Student updateStudent(Long studentId, Student studentDetails) throws ResourceNotFoundException {
        Student student = studentRepository.findById(studentId)
                .orElseThrow(() -> new ResourceNotFoundException("Student id: " + studentId + " not found"));

        student.setFirstName(studentDetails.getFirstName());
        student.setLastName(studentDetails.getLastName());

        Student updatedStudent = studentRepository.save(student);
        return updatedStudent;

    }

    public Map<String, Boolean> deleteStudent(Long studentId) throws ResourceNotFoundException {
        Student student = studentRepository.findById(studentId)
                .orElseThrow(() -> new ResourceNotFoundException("Student id: " + studentId + " not found"));

        studentRepository.delete(student);
        Map<String, Boolean> response = new HashMap<String, Boolean>();
        response.put("deleted", true);
        return response;
    }
}
